module Test.AMR exposing (suite)

import AMR.Main as AMR
import Example.Counter as Example
import Expect exposing (Expectation)
import Fuzz exposing (Fuzzer)
import Help
import Json.Decode
import Json.Encode
import Test exposing (Test, test)


suite : Test
suite =
    Test.describe "AMR"
        [ encodeDecodeEquality
        ]


encodeDecodeEquality : Test
encodeDecodeEquality =
    Test.fuzz exampleModelFuzzer
        "Encoding and decoding a value yields the same value"
        (\model ->
            AMR.encode model
                |> Json.Decode.decodeValue
                    (AMR.decoder (Help.replayUpdate Example.update)
                        Example.msgDecoder
                        0
                    )
                |> Expect.equal (Ok model)
        )



-- Helpers


exampleModelFuzzer : Fuzzer (AMR.Model Example.Model)
exampleModelFuzzer =
    Fuzz.map
        (List.foldl
            (\n model ->
                Tuple.first
                    (AMR.update Example.update
                        (always Cmd.none)
                        Example.encodeMsg
                        (Example.msgFromInt n)
                        model
                    )
            )
            (Tuple.first
                (AMR.init (Help.replayUpdate Example.update)
                    Example.msgDecoder
                    Nothing
                    (Example.init ())
                )
            )
        )
        (Fuzz.list Fuzz.int)
